###############################################################################
# The script below creates the graphics for the Exploring Pharmacokinetic and
# Pharmacodynamic Data chapter of the book Graphics in Life Sciences

# This script has been written in R-2.14.2 and requires the following packages:
require(lattice)
require(grid)
require(ggplot2)
require(reshape)
require(MASS)
require(MSToolkit)

# Working Directory:
# setwd("C:/")
# The following subdirectories are required within the working directory:
# Data            - Containing all datasets to be modelled
# Graphics        - An output directory for graphics

###############################################################################

# Edit lattice and ggplot2 settings for final printing:

# Lattice
latSetList <-  list(
		grid.pars = list(fontfamily = "Helvetica"),
		fontsize = list(text = 12),
		strip.background = list(col = "#DDDDDD"),
		add.text = list(col = "#000000"),
		axis.text = list(cex = 1) ) 


# ggplot2
theme_set(theme_bw(base_size = 12, base_family = "Helvetica"))
theme_update(strip.background = element_rect(fill = "#DDDDDD", colour = "black"),
		strip.text = element_text(family = "Helvetica", size = 12),
		panel.grid.minor = element_line(colour = NA),
		panel.grid.major = element_line(colour = NA),
		panel.border = element_rect(fill = NA, colour = "black"),
		legend.key = element_rect(colour = NA),
		axis.title.x = element_text(family = "Helvetica", size = 12, vjust = 0),
		axis.title.y = element_text(family = "Helvetica", size = 12, vjust = 0.5, angle = 90),
		plot.title = element_text(family = "Helvetica", size = 12 * 1.2, vjust = 1))

###############################################################################
### Data import and manipulation

dataFiles <- list.files("Data")
dataObjects <- gsub(".txt", "", dataFiles)
for(i in 1:length(dataFiles)){
	eval( parse( 
					text = paste(dataObjects[i], " <- read.delim(\"Data/", dataFiles[i], "\", sep = character(0), skip = 1)", sep = "")
			))
}

###############################################################################
# Figures 1, 2 and 3

# DV vs IDV
xpData <- sdtab1
xpData <- subset(xpData, WRES != 0)
names(xpData)[names(xpData) %in% "ID"] <- "Subject"
xpData <- xpData [ do.call("order", xpData[c("Subject", "TIME")]), ]
xpData$Subject <- factor(xpData$Subject )

# Figure 1
figure1 <- qplot(TIME, DV, data = xpData, group = Subject, geom = "line", 
		xlab = "Time after Dose (hour)", ylab = "Observed Concentration (ng/mL)",
		ylim = range(xpData$DV),
		alpha = I(.75))

# Each subjet with a shadow of the whole data
temp <- xpData[xpData$Subject %in% as.character(1:4),]
temp$Subject <- paste("Subject:", temp$Subject)

# Figure 2
figure2 <- qplot(TIME, DV, data = temp, 
				geom = "line", 
				xlab = "Time after Dose (hour)", ylab = "Observed Concentration (ng/mL)",
				#ylim = range(xpData$DV),
				alpha = I(1), size = I(1)) +
		facet_wrap( ~ Subject, scales = "free") +
		geom_point(size = 3, alpha = 1) 



# Each subjet with a shadow of the whole data

# Panel function  for figure 3
pkPanel <- function(x, y, xMain, yMain, ...){
	for(i in unique(xpData$Subject)){
		panel.lines(xMain[xpData$Subject == i], yMain[xpData$Subject == i], col = "#DDDDDD")
		#panel.lines(xMain[xpData$Subject == i], yMain[xpData$Subject == i], col = "#DDDDDDDD") # edited to remove transparency
	}
	panel.lines(x, y, col = "black", lwd=2)
	panel.points(x, y, pch = 16, col = "black", cex = 1)
}

# Strip labelling function for figure 3
stripLabeller <- function(...) strip.default(..., strip.names = T)

# Figure 3
figure3 <- xyplot(DV ~ TIME | Subject, data = xpData[xpData$Subject %in% as.character(1:4),], 
		strip = stripLabeller, 
		panel =  pkPanel, groups = Subject, 
#		main = "Observed Concentration vs Time after Dose by Subject",
		xlab = "Time after Dose (hour)", ylab = "Observed Concentration (ng/mL)",
		ylim = range(xpData$DV),
		scales = list(alternating = 1, tck = c(1, 0)),
		xMain = xpData$TIME, yMain = xpData$DV,
		layout = c(4,1), as.table = T)


###############################################################################
# Figures 4 and 5

set.seed(100)

# DV vs IDV
xpData <- sdtab1
xpData <- subset(xpData, WRES != 0)    # Remove dosing records
xpData <- xpData [ do.call("order", xpData[c("ID", "TIME")]), ]

# Plot of DV vs TIME - Initial look at the data
unused <- xyplot(DV ~ TIME, data = xpData, cex = 1.2, 
#	main = "Observed concentration versus time after dose by subject",
		xlab = "Time after Dose (hour)", ylab = "Observed Concentration (ng/mL)",
		groups = ID, type="b",
		panel = function(x, y, ...) {
			panel.xyplot(x, y, col = "grey40", ...)
			panel.loess(x, y, col = "black", lwd = 4)
		})

# Change in DV by subject
xpData$Diff <- unlist(lapply(split(xpData$DV, xpData$ID), function(x) c(0, diff(x))))
# For diff plot, ensure 'small' changes are set to zero, i.e. no change
xpData$Diff [ abs(xpData$Diff) < .005 * max(xpData$DV) ] <- 0
xpData$Direction <- with(xpData, ifelse( Diff > 0, "Up", ifelse(Diff < 0, "Down", "Same")))

# Count the number of direction changes by subject and merge on to data
nChange <- aggregate(list(N = xpData$Direction), list(ID = xpData$ID), function(x) {
			x <- x [ x != "Same"]
			length(rle(as.character(x))$lengths)
		})
xpData <- merge(xpData, nChange)

# For plot, pool zero and one change and create 3+ category
xpData$N [ xpData$N == 0 ] <- 1
xpData <- xpData [ do.call("order", xpData[c("ID", "TIME")]), ]
xpData$NTRIM <- replace(xpData$N, xpData$N > 3 , 3)
xpData$NFAC <- factor(xpData$NTRIM, levels = 1:3, labels = c("1", "2", "3+"))
xpData <- xpData [ do.call("order", xpData[c("ID", "TIME")]), ]

# Figure 4 - Direction plot of all data
figure4 <- xyplot(DV ~ TIME | NFAC, data = xpData, 
		groups = ID, type = "o", 
		col = "black", 
#	main = "Observed concentration versus time after dose, split by the number of times the sign of the gradient changes direction", 
		xlab = "Time (Hours)", ylab = "Observed Concentration (ng/mL)",
		whichDirection = xpData$Direction, 
		layout = c(3,1),
		panel = function(x, y, groups, subscripts, whichDirection, ...) {
			whichPch <- ifelse(whichDirection == "Up", 24, ifelse(whichDirection == "Down", 25, 22))[subscripts]
			whichCol <- ifelse(whichDirection == "Up", "green", ifelse(whichDirection == "Down", "red", "grey"))[subscripts]
			panel.xyplot(x, y, groups = groups, subscripts = subscripts, type = "l", col="black")
			lpoints(x, y, pch = whichPch, fill = whichCol, col = "black", cex = 1)
		})

# Plot individual subject profiles for subjects with > 2 changes of direction
xpData <- subset(xpData, N > 2)
unused <- xyplot(DV ~ TIME | factor(ID), data = xpData, 
		type = "o", as.table = TRUE, col = "black",
#	main = "Observation vs Time (by Subject indicating changes in sign of gradient)", 
		xlab = "Time (Hours)", ylab = "Observed",
		whichDirection = xpData$Direction, 
		panel = function(x, y, subscripts, whichDirection, ...) {
			whichPch <- ifelse(whichDirection == "Up", 24, ifelse(whichDirection == "Down", 25, 22))[subscripts]
			whichCol <- ifelse(whichDirection == "Up", "green", ifelse(whichDirection == "Down", "red", "grey"))[subscripts]
			panel.xyplot(x, y, type = "l", col="black")
			lpoints(x, y, pch = whichPch, fill = whichCol, col = "black", cex = 1.4)
		})

# Four subject in isolation
whichIds <- c(1, 33, 35, 101)
xpData <- xpData [ xpData$ID %in% whichIds, ]

# Figure 5
figure5 <- xyplot(DV ~ TIME | factor(ID, levels = whichIds, 
				labels = paste("Subject", whichIds)), data = xpData, 
		type = "o", as.table = TRUE, col = "black",
#	main = "Observation Concentration vs Time after dose (by # changes in the sign of the gradient)", 
		xlab = "Time (Hours)", ylab = "Observed Concentration (ng/mL)",
		whichDirection = xpData$Direction, 
		panel = function(x, y, subscripts, whichDirection, ...) {
			whichPch <- ifelse(whichDirection == "Up", 24, ifelse(whichDirection == "Down", 25, 22))[subscripts]
			whichCol <- ifelse(whichDirection == "Up", "green", ifelse(whichDirection == "Down", "red", "grey"))[subscripts]
			panel.xyplot(x, y, type = "l", col="black")
			lpoints(x, y, pch = whichPch, fill = whichCol, col = "black", cex = 1.4)
		})




###############################################################################
# Figures 6 and 7 

######## Scaling Data ###########

xpData <- sdtab1
xpData <- subset(xpData, WRES != 0)
xpData <- xpData [ do.call("order", xpData[c("ID", "TIME")]), ]
nmData <- xpData
pickId <- 36

# Dv over Time
unused <- xyplot(DV ~ TIME, data = nmData, groups = ID, type="o", col="grey60", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				panel.lines(x[flagId], y[flagId], col="red", lwd=2)
				lpoints(x[flagId], y[flagId], pch = 22, col="red", fill="white", cex=3, lwd=2)
				ltext(x[flagId], y[flagId], ids[flagId], col="red", lwd=2)
			}
		}, flagId = nmData$ID == pickId, ids = nmData$ID,
		xlab = "Time (Hours)", ylab = "Observation"
#, main = "DV vs Time"
)

# LogDV over Time
unused <- xyplot(DV ~ TIME, data = nmData, groups = ID, type="o", col="grey60", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				panel.lines(x[flagId], y[flagId], col="red", lwd=2)
				lpoints(x[flagId], y[flagId], pch = 22, col="red", fill="white", cex=3, lwd=2)
				ltext(x[flagId], y[flagId], ids[flagId], col="red", lwd=2)
			}
		}, flagId = nmData$ID == pickId, ids = nmData$ID,
		scales = list(y = list(log = T, at = pretty(nmData$DV))),
		xlab = "Time (Hours)", ylab = "Observation"
#, main = "log(DV) vs Time"
)

# Picking out in-range anomalies
# Distance from the centre of the loess fit
nmData$LogDv <- log(nmData$DV)
fitLo <- with(nmData, loess.smooth(TIME, LogDv))
nmData$CentreLDV <- nmData$LogDv - approx(fitLo$x, fitLo$y, nmData$TIME)$y
unused <- xyplot(CentreLDV ~ TIME, data = nmData, groups = ID, type="o", col="grey", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				panel.lines(x[flagId], y[flagId], col="red", lwd=2)
				lpoints(x[flagId], y[flagId], pch = 22, col="red", fill="white", cex=3, lwd=2)
				ltext(x[flagId], y[flagId], ids[flagId], col="red", lwd=2)
			}
		}, flagId = nmData$ID == pickId, ids = nmData$ID,
		xlab = "Time (Hours)", ylab = "log(DV) - Centre of Loess(log(DV))", 
#main = "'log(DV) - Loess(Log(DV))' vs Time"
)

# Difference from individual LogDV
nmData <- merge(nmData, aggregate(list(MLDV = nmData$CentreLDV), nmData["ID"], mean))
nmData$LDVDiff <- with(nmData, CentreLDV - MLDV)
unused <- xyplot(LDVDiff ~ TIME, data = nmData, groups = ID, type="o", col="grey", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				panel.lines(x[flagId], y[flagId], col="red", lwd=2)
				lpoints(x[flagId], y[flagId], pch = 22, col="red", fill="white", cex=3, lwd=2)
				ltext(x[flagId], y[flagId], ids[flagId], col="red", lwd=2)
			}
		}, flagId = nmData$ID == pickId, ids = nmData$ID,
#main="Rescaled 'LogDv - Centre(Loess(LogDv)) vs Time",
		xlab = "Time (Hours)", ylab = "Rescaled 'LogDv - Centre(Loess(LogDv))")


# Convert to distribution quantiles
splitData <- split(nmData, nmData$TIME)
newData <- do.call("rbind", lapply(splitData, function(df) {
					theMean <- mean(df$LDVDiff )
					theSd <- sd(df$LDVDiff )
					df$ScaleDiff <- with(df, (LDVDiff - theMean)/theSd)
					df
				}))

# Figure 6 
figure6 <- xyplot(ScaleDiff ~ TIME, data = newData, groups = ID, type="o", col="darkgrey", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="darkblue")
			panel.abline(h = -3:3, col = "grey20", lty = 2, lwd=1.2)
		}, flagId = F, ids = newData$ID,
#main="# SDs from Timepoint Centres (LogDv) vs Time",
		xlab = "Time (Hours)", ylab = "# SDs from Timepoint Centres (LogDv)")

# Original plot: flagging all "interesting points" found
unused <- xyplot(DV ~ TIME, data = newData, groups = ID, type="o", col="grey60", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				lpoints(x[flagId], y[flagId], pch = 22, col="red", fill="white", cex=3, lwd=2)
				ltext(x[flagId], y[flagId], ids[flagId], col="red", lwd=2)
			}
		}, flagId = abs(newData$ScaleDiff) > 3, ids = newData$ID,
		xlab = "Time (Hours)", ylab = "Observation"
#,main = "DV vs Time"
)

# Original plot: flagging all "interesting subjects" found
unused <- xyplot(DV ~ TIME, data = newData, groups = ID, type="o", col="grey60", 
		panel = function(x, y, subscripts, groups, flagId, ids, ...) {
			panel.superpose(x, y, subscripts, groups, ...)
			panel.loess(x, y, col="black", lwd=2)
			flagId <- flagId[subscripts]
			ids <- ids[subscripts]
			panel.points(x, y, pch = 15, col="white", cex = 1.5)
			ltext(x, y, ids, col="grey60")
			if(any(flagId)) {
				uniIds <- unique(ids[flagId])
				for (i in uniIds) {
					myTest <- ids == i
					llines(x[myTest], y[myTest], col="red", lwd=2)
					lpoints(x[myTest], y[myTest], pch = 22, col="red", fill="white", cex=3, lwd=2)
					ltext(x[myTest], y[myTest], ids[myTest], col="red", lwd=2)
				}
			}
		}, flagId = abs(newData$ScaleDiff) > 3, ids = newData$ID,
		xlab = "Time (Hours)", ylab = "Observation"
#, main = "DV vs Time"
)

# Flag "interesting subjects" against the overall plot
intId <- unique(newData$ID [ abs(newData$ScaleDiff) > 3 ])
intData <- newData [ newData$ID %in% intId, ]
intData$SUBJ <- factor(paste("Subject", intData$ID), levels = paste("Subject", sort(unique(intData$ID))))
myLower1 <- with(newData, aggregate(list(Lower1 = DV), list(X = TIME), quantile, 0))
myLower2 <- with(newData, aggregate(list(Lower2 = DV), list(X = TIME), quantile, .25))
myUpper1 <- with(newData, aggregate(list(Upper1 = DV), list(X = TIME), quantile, 1))
myUpper2 <- with(newData, aggregate(list(Upper2 = DV), list(X = TIME), quantile, .75))
myMedian  <- with(newData, aggregate(list(Median = DV), list(X = TIME), quantile, .5))
mData <- merge(myLower1, merge(myLower2, merge(myMedian, merge(myUpper1, myUpper2))))
mData$X <- as.numeric(as.character(mData$X))

# Figure 7
figure7 <- xyplot(
		DV ~ TIME | SUBJ, 
		data = intData, 
		layout = c(length(intId), 1),
		panel = function(x, y, mData) {
			with(mData, {
						lpolygon( c(X, rev(X)), c(Lower1, rev(Upper1)), col = "skyblue2", border = F)
						lpolygon( c(X, rev(X)), c(Lower2, rev(Upper2)), col = "skyblue4", border = F)
						llines(X, Median, col = "black", lwd = 2)
					})
			llines(x, y, col="red", lwd = 2)
			lpoints(x, y, col="black", pch = 21, fill = "red", cex = 1, lwd = 2)
		}, 
		mData = mData, 
		ylim = c(0, 170),
		ylab = "Observed Concentration (ng/mL)", 
		xlab = "Time after dose (Hours)", 
#	main = "Observed Concentration vs Time after dose",
		scales = list(y = list( at = c(0, 30, 60, 90, 120, 150) ) ) 
)

###############################################################################
# Figures 8 and 9

set.seed(5234524)

# Generate some baseline values and allocate some doses
baselines <- round(runif(200, 20, 80),2)
resultsData <- data.frame(id = 1:200, w0 = baselines)
resultsData$dose <- sample(0:2, 200, T)

# Create a time trend with increasing variability
nRow <- nrow(resultsData)
resultsData$w1 <- resultsData$w0 + jitter(resultsData$dose/4) + rnorm(nRow, sd = 1.1)
resultsData$w2 <- resultsData$w1 + jitter(resultsData$dose/4) + rnorm(nRow, sd = 1.3)
resultsData$w4 <- resultsData$w2 + jitter(resultsData$dose/5) + rnorm(nRow, sd = 2.0)
resultsData$w6 <- resultsData$w4 + jitter(resultsData$dose/7) + rnorm(nRow, sd = 2.2)
resultsData$w8 <- resultsData$w6 + jitter(resultsData$dose/8) + rnorm(nRow, sd = 2.4)
resultsData$w10 <- resultsData$w8 + jitter(resultsData$dose/9) + rnorm(nRow, sd = 2.6)

# Restructure the data for plotting
resultsMelt <- melt(resultsData, id = c("id", "dose"))
resultsMelt$time <- as.numeric(gsub("w", "", resultsMelt$variable))
resultsMelt$dose <- factor(resultsMelt$dose)


# Calculate change from baseline
changeBase <- resultsData
changeBase[, 4:9] <- changeBase[, 4:9] - changeBase[,2]

# Restructure the data for plotting
changeMelt <- melt(changeBase, id = c("id", "dose"))
changeMelt$time <- as.numeric(gsub("w", "", changeMelt$variable))
changeMelt$dose <- factor(changeMelt$dose)
changeMelt <- changeMelt[changeMelt$time>0,]

# Figure 8 - Baseline Adjusted Plot
figure8a <- qplot(time, value, data = resultsMelt, group = id, geom = "line", alpha = I(.5), ylab = "DV", ylim = c(0,100), xlab = "Week") 
figure8b <- qplot(time, value, data = changeMelt, group = id, geom = "line", alpha = I(.5), ylab = "Change from Baseline in DV", xlab = "Week") 


# Allocate some somejects to countries randomly
demoData1 <- createDiscreteCovariates(subjects = 200, names = c("Sex", "Smokes", "Country"), 
		values = "F,M#Y,N#UK,USA,France,Italy,Germany,Japan,China,Chile,Mexico",
		probs = ".4,.6#.3,.7#.14,.31,.1,.09,.08,.07,.07,.07,.07")

changeMeltFull <- merge(demoData1, changeMelt, by.x = "SUBJ", by.y = "id")
changeMeltFull <- changeMeltFull[order(changeMeltFull$SUBJ, changeMeltFull$time),]

countryProfiles <- aggregate(changeMeltFull$value, list(changeMeltFull$Country,changeMeltFull$time), median)
names(countryProfiles) <- c("Country", "Week", "y")

# Figure 9
figure9 <- qplot(x = Week, y = y, data = countryProfiles, geom = "line", colour = Country, size = I(1.4)) +
		labs(x = "Week", y = "Change from baseline in response variable by country")


###############################################################################
# Figure 10

# Calculate quantiles for change from baseline data
# Use quantiles to create a dataset for drawing polygons

uniTimes <- sort(unique(changeMelt$time))
polyTimes <- c(uniTimes,rev(uniTimes)) 
minMaxPoly <- c(tapply(changeMelt$value, changeMelt$time, min), rev(tapply(changeMelt$value, changeMelt$time, max)))
quartilePoly <- c(tapply(changeMelt$value, changeMelt$time, quantile, prob = .25), rev(tapply(changeMelt$value, changeMelt$time, quantile, prob = .75)))
names(minMaxPoly) <- NULL
names(quartilePoly) <- NULL
allData <- data.frame(Week = polyTimes, yAll = minMaxPoly)
allData <- rbind(allData, allData, allData)


polyData1 <- aggregate(changeMelt$value, list(changeMelt$time, changeMelt$dose), quantile, prob = c(0))
polyData2 <- aggregate(changeMelt$value, list(changeMelt$time, changeMelt$dose), quantile, prob = c(.25))
polyData3 <- aggregate(changeMelt$value, list(changeMelt$time, changeMelt$dose), quantile, prob = c(.75))
polyData4 <- aggregate(changeMelt$value, list(changeMelt$time, changeMelt$dose), quantile, prob = c(1))
polyData3 <- polyData3[nrow(polyData3):1,]
polyData4 <- polyData4[nrow(polyData4):1,]
polyData1  <- cbind(polyData1, x2 = polyData2[,3])
polyData3  <- cbind(polyData4, x2 = polyData3[,3]) 
polyData <- rbind(polyData1, polyData3)
names(polyData) <- c("Week", "Dose", "y1", "y2")
lineData <- aggregate(changeMelt$value, list(changeMelt$time, changeMelt$dose), median)
names(lineData) <- c("Week", "Dose", "median")
polyData <- polyData[order(polyData$Dose), ]
polyData <- cbind(polyData, yAll = allData[,2])

figure10 <- qplot(x = Week, y = yAll, data = polyData, geom = "polygon", fill = I("grey")) + #,
#				main = "Change in DV over Time by Dose Group") +  
		facet_grid(. ~ Dose, labeller = label_both) + 
		geom_polygon(data = polyData, aes(y = y1), fill = "skyblue2") +
		geom_polygon(data = polyData, aes(y = y2), fill = "skyblue4") +
		geom_line(data = lineData, aes(y = median), size = 1.4) +
		theme(panel.margin = unit(0, "lines")) +
		labs(y = "Response", x = "Week")


###############################################################################
# Figure 11 - Covariate groupings

xpData <- sdtab1
cov1 <- cotab1
cov2 <- catab1
covData <- cbind(cov1, cov2[ , setdiff(names(cov2), names(cov1)), drop = FALSE])
xpData <- cbind(xpData, covData[, setdiff(names(covData), names(xpData)), drop = FALSE])
xpData <- subset(xpData, WRES != 0)[c("ID", "TIME", "DV", "AGE", "SEX", "SMOK", "HT", "WT")]

grpPanel <- function(x, y, allX, allLower, allUpper, ids, subscripts) {
	whichBlue <- "skyblue"
	
	ids <- ids [ subscripts ]
	lpolygon( c(allX, rev(allX)), c(allLower, rev(allUpper)), col = "grey", border = F)
	uniX <- sort(unique(x))
	
	uniLower <- tapply(y, x, min)
	uniUpper <- tapply(y, x, max)
	lpolygon( c(uniX, rev(uniX)), c(uniLower, rev(uniUpper)), col = paste(whichBlue, 2, sep=""), border = F)
	
	uniLower <- tapply(y, x, quantile, .05)
	uniUpper <- tapply(y, x, quantile, .95)
	lpolygon( c(uniX, rev(uniX)), c(uniLower, rev(uniUpper)), col = paste(whichBlue, 4, sep=""), border = F)
	
	llines(uniX, tapply(y, x, median), lwd = 3, col = "black")
	
	ltext(12, 155, paste(length(unique(ids)), "subjects"), font = 3, adj = c(1, 0.5))
}

# AGE
xpData$AgeFactor <- with(xpData, cut(AGE, c(23, 40, 50, 60, 100)))
unused <- xyplot(DV ~ TIME | AgeFactor, data = xpData, layout = c(4,1), 
		allX = sort(unique(xpData$TIME)), 
		allLower = tapply(xpData$DV, xpData$TIME, min), 
		allUpper = tapply(xpData$DV, xpData$TIME, max), 
		panel = grpPanel, subset = order(xpData$TIME), strip = strip.custom(strip.names = TRUE, var.name = "Age"), ids = xpData$ID,
#	main = "Observed Concentration vs Time after Dose (by Age Group)", 
		xlab = "Time after dose (Hour)", ylab = "Observed Concentration (ng/mL)"
)

# WT
xpData$WtFactor <- with(xpData, cut(WT, c(50, 75, 85, 95, 140)))
unused <- xyplot(DV ~ TIME | WtFactor, data = xpData, layout = c(4,1), 
		allX = sort(unique(xpData$TIME)), 
		allLower = tapply(xpData$DV, xpData$TIME, min), 
		allUpper = tapply(xpData$DV, xpData$TIME, max), 
		panel = grpPanel, subset = order(xpData$TIME), strip = strip.custom(strip.names = TRUE, var.name = "Weight"), ids = xpData$ID,
#	main = "Observed Concentration vs Time after Dose (by Weight Group)", 
		xlab = "Time after dose (Hour)", ylab = "Observed Concentration (ng/mL)"
)


# HT
xpData$HtFactor <- with(xpData, cut(HT, c(135, 160, 172, 178, 190)))
unused <- xyplot(DV ~ TIME | HtFactor, data = xpData, layout = c(4,1), 
		allX = sort(unique(xpData$TIME)), 
		allLower = tapply(xpData$DV, xpData$TIME, min), 
		allUpper = tapply(xpData$DV, xpData$TIME, max), 
		panel = grpPanel, subset = order(xpData$TIME), strip = strip.custom(strip.names = TRUE, var.name = "Height"), ids = xpData$ID,
#	main = "Observed Concentration vs Time after Dose (by Height Group)", 
		xlab = "Time after dose (Hour)", ylab = "Observed Concentration (ng/mL)"
)

# SEX/SMOK
xpData$SEX.SMOK <- paste(ifelse(xpData$SEX == 1, "Male  ", "Female"), ifelse(xpData$SMOK == 0, "\nNon Smoker", "\nSmoker       "), "      ", sep = "")
figure11 <- xyplot(DV ~ TIME | SEX.SMOK, data = xpData, layout = c(4,1), 
		allX = sort(unique(xpData$TIME)), 
		allLower = tapply(xpData$DV, xpData$TIME, min), 
		allUpper = tapply(xpData$DV, xpData$TIME, max), 
		panel = grpPanel, subset = order(xpData$TIME), 
		ids = xpData$ID,
		par.strip.text = list(lines=3),
#	main = "Observed Concentration vs Time after Dose (by Sex/Smoking Indicator)", 
		xlab = "Time after dose (Hour)", ylab = "Observed Concentration (ng/mL)"
)



# Simulate subject weights
Weights <- createContinuousCovariates(200, "Weight", 60, 30, range = "40 < Weight < 100", idCol = "id")
changeMeltNew <- merge(Weights, changeMelt)
names(changeMeltNew)[names(changeMeltNew) == "time"] <- "Week"

figure12 <- qplot(Weight, value, data = changeMeltNew[changeMeltNew$Week != 1,], fill = dose, shape = dose) + #,
		facet_grid(~ Week, labeller = label_both) +
		scale_shape_manual(values = c(21, 22, 24)) +
		scale_fill_manual(values = c("black", "skyblue2", "red")) +
		theme(panel.margin = unit(0, "lines")) + 
		labs(y = "Change from Baseline in response variable", fill = "Dose", shape = "Dose", x = "Weight")


###############################################################################
# Output All Graphics to a PDF

pdf("graphics/output.pdf", paper = "a4r", height = 8, width = 12)
trellis.par.set(latSetList)	# Lattice theme

print(figure1)
print(figure2)
print(figure3)
print(figure4)
print(figure5)
print(figure6)
print(figure7)

grid.newpage()
pushViewport(viewport(layout = grid.layout(
						nrow = 2, 
						ncol = 2, 
						widths = unit(c(1,1), c("null", "null")),
						heights = unit(c(1,1), c("lines", "null"))
				)))
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1))
#grid.text("Improved Clarity After Adjusting for Baseline", 1, .5, gp = gpar(cex = 1.4))
popViewport()
pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2))
print(figure8a, newpage = F)
popViewport()
pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 2))
print(figure8b, newpage = F)
popViewport()

print(figure9)
print(figure10)
print(figure11)
print(figure12)


dev.off()